%% demo_SUNSAL
%
%  SUNSAL -> sparse unmixing via variable splitting and augmented
%  Lagrangian methods
%
%% --------------- Description --------------------------------------------
%
%  SUNSAL solves the following l2-l1 optimization  problem
%  [size(M) = (L,p); size(X) = (p,N)]; size(Y) = (L,N)]
%
%         min  (1/2) ||M X-y||^2_F + lambda ||X||_1
%          X
%
%  where ||X||_1 = sum(sum(abs(X)).
%
%    CONSTRAINTS ACCEPTED:
%
%    1) POSITIVITY:  X >= 0;
%    2) ADDONE:  sum(X) = ones(1,N);
%
%    NOTES:
%       1) The optimization w.r.t each column of X is decoupled. Thus,
%          SUNSAL solves N simultaneous problems.
%
%       2) SUNSAL solves the following  problems:
%
%          a) BPDN - Basis pursuit denoising l2-l1
%                    (lambda > 0, POSITIVITY = 'no', ADDONE, 'no')
%
%          b) CBPDN - Constrained basis pursuit denoising l2-l1
%                    (lambda > 0, POSITIVITY = 'yes', ADDONE, 'no')
%
%          c) CLS   - Constrained least squares
%                     (lambda = 0, POSITIVITY = 'yes', ADDONE, 'no')
%
%          c) FCLS   - Fully constrained least squares
%                     (lambda >=0 , POSITIVITY = 'yes', ADDONE, 'yes')
%                      In this case, the regularizer ||X||_1  plays no role,
%                      as it is constant.
%
%
%
%
%
% Author: Jose Bioucas-Dias, 2010
%
% -----------------------------------------------------------------------

clear all
close all
%%  -------------  Simulation parameters
%
%--------------------------------------------------------------------------
% Algorithm
%--------------------------------------------------------------------------
% 1 - BPDN
% 2 - CBPDN
% 3 - CLS
% 4 - FCLS
algorithm = 2;

%
%--------------------------------------------------------------------------
% Load Library (difficulty degree: 4 > 3 > 2  \simeq  1)
%--------------------------------------------------------------------------
% 1 - iid  Gaussian
% 2 - iid Uniform
% 3 - USGS   - pruned
% 4 - USGS
library = 4;

% Number of spectral banda (needed just for library \in {1,2}
% L = 224 for library \in {3,4})
L = 224;

% Number of spectral signatures
p = 10;

% Number of pixels
N = 100;

% Number of active columns (k <= p). Simulate a k-sparse signal.
k = 5;

if k > p
    error('number of active columns larger than the number of columns')
end

%set SNR in dBs  (SNR = ||Mx||^2_F / ||noise||^2_F)
SNR = 40;


% define random states
rand('state',10);
randn('state',10);


%--------------------------------------------------------------------------
% Start simulation
%--------------------------------------------------------------------------

switch library
    case 1
        A = randn(L,p);
    case 2
        A = rand(L,p);
    case 3
        load USGS_pruned_10_deg.mat
        A = B;
        clear B;
        % select at random p materials
        indexes = randperm(size(A,2));
        A = A(:,indexes(1:p));
    case 4
        load USGS_1995_Library.mat
        wavelengths = datalib(:,1);
        [dummy, indexes] = sort(wavelengths);
        A = datalib(indexes,4:end);
        names = names(4:end,:);
        clear datalib;
        % select at random p materials
        indexes = randperm(size(A,2));
        A = A(:,indexes(1:p));
    otherwise
        disp('Unknown library')
end

[L,p] = size(A);  % L = number of bands; p = number of materials



%---------------------------------
% generate  original data x
%---------------------------------
x = zeros(p,N);
indexes = randperm(p);
% generate fractional abundances
x(indexes(1:k),:) = dirichlet(ones(k,1),N)';

%---------------------------------
% generate  the observed  data x
%---------------------------------

% set noise standard deviation
sigma = sqrt(sum(sum((A*x).^2))/N/L/10^(SNR/10));

noise = sigma*randn(L,N);
% observed spectral vector
y = A*x + noise;



%--------------------------------------------------------------------------
% RUN SUNSAL
%--------------------------------------------------------------------------

switch algorithm
    case 1
        % BPDN
        lambda = 2e-3;
        x_hat = sunsal(A,y,'lambda',lambda,'ADDONE','no','POSITIVITY','no', ...
            'AL_iters',200,'TOL', 1e-4,'verbose','yes');
    case 2
        %CBPDN
        lambda = 2e-3;
        x_hat = sunsal(A,y,'lambda',lambda,'ADDONE','no','POSITIVITY','yes', ...
            'AL_iters',200,'TOL', 1e-4, 'verbose','yes');
    case 3
        %CLS
        lambda = 0.0;
        x_hat = sunsal(A,y,'lambda',lambda,'ADDONE','no','POSITIVITY','yes', ...
            'AL_iters',200,'TOL', 1e-4,'verbose','yes');
    case 4
        % FCLN
        lambda = 0;
        x_hat = sunsal(A,y,'lambda',lambda,'ADDONE','yes','POSITIVITY','yes', ...
            'AL_iters',200,'TOL', 1e-4, 'verbose','yes');

end



%--------------------------------------------------------------------------
% SUNSAL - plot results
%--------------------------------------------------------------------------


xx = 1:p;
% RMSE
fprintf('RMSE = %f\n\n', norm(x_hat-x,'fro'));
% signal-to-reconstruction error
fprintf('SRE (db) = %f\n\n', 10*(log10(sum(x(:).^2)./sum((x_hat(:)-x(:)).^2))));


figure(1)
imagesc(x)
set(gcf,'Position', [1 600 560   420])
title('Original X')
xlabel('pixel')
ylabel('endmeber')

figure(2)
imagesc(x_hat)
set(gcf,'Position', [600 600 560   420])
title('Estimated X')
xlabel('pixel')
ylabel('endmeber')





